@@ -52,7 +52,7 @@ totem_action_info (char *reason, Totem *totem)
 	if (totem == NULL)
 		parent = NULL;
 	else
-		parent = totem->win;
+		parent = totem->prefs;
 
 	error_dialog =
 		gtk_message_dialog_new (GTK_WINDOW (parent),
@@ -383,7 +383,6 @@ audio_out_menu_changed (GtkComboBox *combobox, Totem *totem)
 					"restarted."),
 				totem);
 	}
-
 }
 
 static void
@@ -441,6 +440,8 @@ totem_setup_preferences (Totem *totem)
 
 	totem->prefs = glade_xml_get_widget (totem->xml,
 			"totem_preferences_window");
+	gtk_window_set_transient_for (GTK_WINDOW (totem->prefs),
+			GTK_WINDOW (totem->win));
 
 	g_signal_connect (G_OBJECT (totem->prefs), "response",
 			G_CALLBACK (gtk_widget_hide), NULL);
